@@ -301,7 +301,8 @@ void QAspectEnginePrivate::shutdown()
 
 void QAspectEnginePrivate::exitSimulationLoop()
 {
-    m_aspectManager->exitSimulationLoop();
+    if (m_aspectManager != nullptr)
+        m_aspectManager->exitSimulationLoop();
 }
 
 /*!
@@ -521,3 +522,5 @@ QAspectEngine::RunMode QAspectEngine::runMode() const
 } // namespace Qt3DCore
 
 QT_END_NAMESPACE
+
+#include "moc_qaspectengine.cpp"
