@@ -29,12 +29,13 @@
 #include <QtTest/QTest>
 #include <Qt3DRender/qrendertargetoutput.h>
 #include <Qt3DRender/private/uniform_p.h>
+#include <graphicshelpergl3_2_p.h>
 #include <Qt3DRender/private/attachmentpack_p.h>
-#include <graphicshelpergl3_3_p.h>
 #include <QOpenGLBuffer>
-#include <QOpenGLFunctions_3_3_Core>
+#include <QOpenGLFunctions_3_2_Core>
 #include <QOpenGLShaderProgram>
 #include <QOpenGLVertexArrayObject>
+#include <QSurfaceFormat>
 
 #if !defined(QT_OPENGL_ES_2) && defined(QT_OPENGL_3_2)
 
@@ -47,7 +48,7 @@ using namespace Qt3DRender::Render::OpenGL;
 namespace {
 
 const QByteArray vertCode = QByteArrayLiteral(
-            "#version 330 core\n" \
+            "#version 150 core\n" \
             "in vec3 vertexPosition;\n" \
             "in vec2 vertexTexCoord;\n" \
             "out vec2 texCoord;\n" \
@@ -58,7 +59,7 @@ const QByteArray vertCode = QByteArrayLiteral(
             "}\n");
 
 const QByteArray vertCodeUniformBuffer = QByteArrayLiteral(
-            "#version 330 core\n" \
+            "#version 150 core\n" \
             "in vec3 vertexPosition;\n" \
             "in vec2 vertexTexCoord;\n" \
             "in int vertexColorIndex;\n" \
@@ -72,7 +73,7 @@ const QByteArray vertCodeUniformBuffer = QByteArrayLiteral(
             "}\n");
 
 const QByteArray fragCodeFragOutputs = QByteArrayLiteral(
-            "#version 330 core\n" \
+            "#version 150 core\n" \
             "out vec4 color;\n" \
             "out vec2 temp;\n" \
             "void main()\n" \
@@ -82,7 +83,7 @@ const QByteArray fragCodeFragOutputs = QByteArrayLiteral(
             "}\n");
 
 const QByteArray fragCodeUniformsFloat = QByteArrayLiteral(
-            "#version 330 core\n" \
+            "#version 150 core\n" \
             "out vec4 color;\n" \
             "uniform float multiplier;\n" \
             "uniform vec2 multiplierVec2;\n" \
@@ -95,8 +96,8 @@ const QByteArray fragCodeUniformsFloat = QByteArrayLiteral(
             "}\n");
 
 const QByteArray fragCodeUniformsInt = QByteArrayLiteral(
-            "#version 330 core\n" \
-            "out ivec4 color;\n" \
+            "#version 150 core\n" \
+            "out vec4 color;\n" \
             "uniform int multiplier;\n" \
             "uniform ivec2 multiplierVec2;\n" \
             "uniform ivec3 multiplierVec3;\n" \
@@ -108,8 +109,8 @@ const QByteArray fragCodeUniformsInt = QByteArrayLiteral(
             "}\n");
 
 const QByteArray fragCodeUniformsUInt = QByteArrayLiteral(
-            "#version 330 core\n" \
-            "out uvec4 color;\n" \
+            "#version 150 core\n" \
+            "out vec4 color;\n" \
             "uniform uint multiplier;\n" \
             "uniform uvec2 multiplierVec2;\n" \
             "uniform uvec3 multiplierVec3;\n" \
@@ -121,7 +122,7 @@ const QByteArray fragCodeUniformsUInt = QByteArrayLiteral(
             "}\n");
 
 const QByteArray fragCodeUniformsFloatMatrices = QByteArrayLiteral(
-            "#version 330 core\n" \
+            "#version 150 core\n" \
             "out vec4 color;\n" \
             "uniform mat2 m2;\n" \
             "uniform mat2x3 m23;\n" \
@@ -139,7 +140,7 @@ const QByteArray fragCodeUniformsFloatMatrices = QByteArrayLiteral(
             "}\n");
 
 const QByteArray fragCodeUniformBuffer = QByteArrayLiteral(
-            "#version 330 core\n" \
+            "#version 150 core\n" \
             "out vec4 color;\n" \
             "in vec2 texCoord;\n" \
             "flat in int colorIndex;\n" \
@@ -153,7 +154,7 @@ const QByteArray fragCodeUniformBuffer = QByteArrayLiteral(
             "}\n");
 
 const QByteArray fragCodeSamplers = QByteArrayLiteral(
-            "#version 330 core\n" \
+            "#version 150 core\n" \
             "in vec2 texCoord;\n" \
             "out vec4 color;\n" \
             "uniform sampler1D s1;\n" \
@@ -175,7 +176,7 @@ const QByteArray fragCodeSamplers = QByteArrayLiteral(
 
 } // anonymous
 
-class tst_GraphicsHelperGL3_3 : public QObject
+class tst_GraphicsHelperGL3_2 : public QObject
 {
     Q_OBJECT
 private Q_SLOTS:
@@ -185,9 +186,10 @@ private Q_SLOTS:
         m_window.reset(new QWindow);
         m_window->setSurfaceType(QWindow::OpenGLSurface);
         m_window->setGeometry(0, 0, 10, 10);
+        m_window->create();
 
         QSurfaceFormat format;
-        format.setVersion(3, 3);
+        format.setVersion(3, 2);
         format.setProfile(QSurfaceFormat::CoreProfile);
         format.setDepthBufferSize(24);
         format.setSamples(4);
@@ -195,19 +197,17 @@ private Q_SLOTS:
         m_window->setFormat(format);
         m_glContext.setFormat(format);
 
-        m_window->create();
-
         if (!m_glContext.create()) {
             qWarning() << "Failed to create OpenGL context";
             return;
         }
 
         if (!m_glContext.makeCurrent(m_window.data())) {
-            qWarning() << "Failed to maed OpenGL context current";
+            qWarning() << "Failed to make OpenGL context current";
             return;
         }
 
-        if ((m_func = m_glContext.versionFunctions<QOpenGLFunctions_3_3_Core>()) != nullptr) {
+        if ((m_func = m_glContext.versionFunctions<QOpenGLFunctions_3_2_Core>()) != nullptr) {
             m_glHelper.initializeHelper(&m_glContext, m_func);
             m_initializationSuccessful = true;
         }
@@ -221,14 +221,14 @@ private Q_SLOTS:
     void alphaTest()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
         // Deprecated
     }
 
     void bindBufferBase()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLuint bufferId = 0;
@@ -256,7 +256,7 @@ private Q_SLOTS:
     void bindFragDataLocation()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -282,155 +282,57 @@ private Q_SLOTS:
     void bindFrameBufferAttachment()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
-        {
-            // GIVEN
-            GLuint fboId;
-            m_func->glGenFramebuffers(1, &fboId);
+        // GIVEN
+        GLuint fboId;
+        m_func->glGenFramebuffers(1, &fboId);
 
-            Attachment attachment;
-            attachment.m_point = QRenderTargetOutput::Color0;
+        Attachment attachment;
+        attachment.m_point = QRenderTargetOutput::Color0;
 
-            GLint maxAttachmentsCount = 0;
-            m_func->glGetIntegerv(GL_MAX_COLOR_ATTACHMENTS, &maxAttachmentsCount);
+        // THEN
+        QVERIFY(fboId != 0);
 
-            // THEN
-            QVERIFY(fboId != 0);
-            QVERIFY(maxAttachmentsCount >= 3);
+        // WHEN
+        m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, fboId);
 
-            // WHEN
-            m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, fboId);
+        QOpenGLTexture texture(QOpenGLTexture::Target2D);
+        texture.setSize(512, 512);
+        texture.setFormat(QOpenGLTexture::RGBA32F);
+        texture.setMinificationFilter(QOpenGLTexture::Linear);
+        texture.setMagnificationFilter(QOpenGLTexture::Linear);
+        texture.setWrapMode(QOpenGLTexture::ClampToEdge);
+        if (!texture.create())
+            qWarning() << "Texture creation failed";
+        texture.allocateStorage();
+        QVERIFY(texture.isStorageAllocated());
+        GLint error = m_func->glGetError();
+        QVERIFY(error == 0);
+        m_glHelper.bindFrameBufferAttachment(&texture, attachment);
 
-            QOpenGLTexture texture(QOpenGLTexture::Target2D);
-            texture.setSize(512, 512);
-            texture.setFormat(QOpenGLTexture::RGBA8U);
-            texture.setMinificationFilter(QOpenGLTexture::Linear);
-            texture.setMagnificationFilter(QOpenGLTexture::Linear);
-            texture.setWrapMode(QOpenGLTexture::ClampToEdge);
-            if (!texture.create())
-                qWarning() << "Texture creation failed";
-            texture.allocateStorage();
-            QVERIFY(texture.isStorageAllocated());
-            GLint error = m_func->glGetError();
-            QVERIFY(error == 0);
-            m_glHelper.bindFrameBufferAttachment(&texture, attachment);
+        // THEN
+        GLenum status = m_func->glCheckFramebufferStatus(GL_DRAW_FRAMEBUFFER);
+        QVERIFY(status == GL_FRAMEBUFFER_COMPLETE);
 
-            // THEN
-            GLenum status = m_func->glCheckFramebufferStatus(GL_DRAW_FRAMEBUFFER);
-            QCOMPARE(int(status), GL_FRAMEBUFFER_COMPLETE);
+        error = m_func->glGetError();
+        QVERIFY(error == 0);
+        GLint textureAttachmentId = 0;
+        m_func->glGetFramebufferAttachmentParameteriv(GL_DRAW_FRAMEBUFFER,
+                                                      GL_COLOR_ATTACHMENT0,
+                                                      GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME,
+                                                      &textureAttachmentId);
+        QCOMPARE(GLuint(textureAttachmentId), texture.textureId());
 
-            error = m_func->glGetError();
-            QVERIFY(error == 0);
-            GLint textureAttachmentId = 0;
-            m_func->glGetFramebufferAttachmentParameteriv(GL_DRAW_FRAMEBUFFER,
-                                                          GL_COLOR_ATTACHMENT0,
-                                                          GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME,
-                                                          &textureAttachmentId);
-            QCOMPARE(GLuint(textureAttachmentId), texture.textureId());
-
-            // Restore state
-            m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
-            m_func->glDeleteFramebuffers(1, &fboId);
-        }
-        {
-            // GIVEN
-            QOpenGLTexture texture(QOpenGLTexture::TargetCubeMap);
-            texture.setSize(512, 512);
-            texture.setFormat(QOpenGLTexture::RGBA32F);
-            texture.setMinificationFilter(QOpenGLTexture::Linear);
-            texture.setMagnificationFilter(QOpenGLTexture::Linear);
-            texture.setWrapMode(QOpenGLTexture::ClampToEdge);
-            if (!texture.create())
-                qWarning() << "Texture creation failed";
-            texture.allocateStorage();
-            QVERIFY(texture.isStorageAllocated());
-            GLint error = m_func->glGetError();
-            QVERIFY(error == 0);
-
-            { // Check All Faces
-
-                // GIVEN
-                GLuint fboId;
-                m_func->glGenFramebuffers(1, &fboId);
-
-                // THEN
-                QVERIFY(fboId != 0);
-
-                // WHEN
-                m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, fboId);
-
-                Attachment attachment;
-                attachment.m_point = QRenderTargetOutput::Color0;
-                attachment.m_face = Qt3DRender::QAbstractTexture::AllFaces;
-
-                m_glHelper.bindFrameBufferAttachment(&texture, attachment);
-
-                // THEN
-                GLenum status = m_func->glCheckFramebufferStatus(GL_DRAW_FRAMEBUFFER);
-                QVERIFY(status == GL_FRAMEBUFFER_COMPLETE);
-
-                error = m_func->glGetError();
-                QVERIFY(error == 0);
-                GLint textureIsLayered = 0;
-                m_func->glGetFramebufferAttachmentParameteriv(GL_DRAW_FRAMEBUFFER,
-                                                              GL_COLOR_ATTACHMENT0,
-                                                              GL_FRAMEBUFFER_ATTACHMENT_LAYERED,
-                                                              &textureIsLayered);
-                QCOMPARE(textureIsLayered, GL_TRUE);
-
-                // Restore state
-                m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
-                m_func->glDeleteFramebuffers(1, &fboId);
-            }
-            { // Check Specific Faces
-
-                // GIVEN
-                GLuint fboId;
-                m_func->glGenFramebuffers(1, &fboId);
-
-                // THEN
-                QVERIFY(fboId != 0);
-
-                // WHEN
-                m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, fboId);
-
-                Attachment attachment;
-                attachment.m_point = QRenderTargetOutput::Color0;
-                attachment.m_face = Qt3DRender::QAbstractTexture::CubeMapNegativeZ;
-
-                m_glHelper.bindFrameBufferAttachment(&texture, attachment);
-
-                // THEN
-                GLenum status = m_func->glCheckFramebufferStatus(GL_DRAW_FRAMEBUFFER);
-                QVERIFY(status == GL_FRAMEBUFFER_COMPLETE);
-
-                error = m_func->glGetError();
-                QVERIFY(error == 0);
-                GLint textureFace = 0;
-                m_func->glGetFramebufferAttachmentParameteriv(GL_DRAW_FRAMEBUFFER,
-                                                              GL_COLOR_ATTACHMENT0,
-                                                              GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE,
-                                                              &textureFace);
-                QCOMPARE(textureFace, GL_TEXTURE_CUBE_MAP_NEGATIVE_Z);
-                GLint textureIsLayered = 0;
-                m_func->glGetFramebufferAttachmentParameteriv(GL_DRAW_FRAMEBUFFER,
-                                                              GL_COLOR_ATTACHMENT0,
-                                                              GL_FRAMEBUFFER_ATTACHMENT_LAYERED,
-                                                              &textureIsLayered);
-                QCOMPARE(textureIsLayered, GL_FALSE);
-
-                // Restore state
-                m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
-                m_func->glDeleteFramebuffers(1, &fboId);
-            }
-        }
+        // Restore state
+        m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
+        m_func->glDeleteFramebuffers(1, &fboId);
     }
 
     void bindFrameBufferObject()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLuint fboId;
@@ -476,14 +378,14 @@ private Q_SLOTS:
     void bindShaderStorageBlock()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
-        // Not supported in OpenGL 3.3
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
+        // Not supported in OpenGL 3.2
     }
 
     void bindUniformBlock()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -503,7 +405,7 @@ private Q_SLOTS:
     void blendEquation()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLint equation = 0;
@@ -521,22 +423,22 @@ private Q_SLOTS:
     void blendFunci()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
-        // Not supported by OpenGL 3.3
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
+        // Not supported by OpenGL 3.2
     }
 
     void blendFuncSeparatei()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
-        // Not supported by OpenGL 3.3
+        // Not supported by OpenGL 3.2
     }
 
     void boundFrameBufferObject()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLuint fboId;
@@ -561,7 +463,7 @@ private Q_SLOTS:
     void checkFrameBufferComplete()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
         // GIVEN
         GLuint fboId;
         m_func->glGenFramebuffers(1, &fboId);
@@ -594,7 +496,7 @@ private Q_SLOTS:
     void clearBufferf()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         // GIVEN
@@ -650,7 +552,6 @@ private Q_SLOTS:
         textures[3]->bind();
         m_func->glGetTexImage(GL_TEXTURE_2D, 0, GL_RGBA, GL_FLOAT, colors.data());
         textures[3]->release();
-
         for (const QVector4D c : colors) {
             QVERIFY(c == clearValue1);
         }
@@ -676,7 +577,7 @@ private Q_SLOTS:
     void createFrameBufferObject()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // WHEN
         const GLuint fboId = m_glHelper.createFrameBufferObject();
@@ -691,7 +592,7 @@ private Q_SLOTS:
     void depthMask()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLboolean depthWritingEnabled = false;
@@ -718,7 +619,7 @@ private Q_SLOTS:
     void depthTest()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisable(GL_DEPTH_TEST);
@@ -743,7 +644,7 @@ private Q_SLOTS:
     void disableClipPlane()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glEnable(GL_CLIP_DISTANCE0 + 5);
@@ -761,7 +662,7 @@ private Q_SLOTS:
     void disablei()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glEnablei(GL_BLEND, 2);
@@ -779,7 +680,7 @@ private Q_SLOTS:
     void disablePrimitiveRestart()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glEnable(GL_PRIMITIVE_RESTART);
@@ -797,7 +698,7 @@ private Q_SLOTS:
     void drawBuffers()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLuint fboId;
@@ -869,7 +770,7 @@ private Q_SLOTS:
     void enableClipPlane()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisable(GL_CLIP_DISTANCE0 + 4);
@@ -887,7 +788,7 @@ private Q_SLOTS:
     void enablei()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisablei(GL_BLEND, 4);
@@ -905,7 +806,7 @@ private Q_SLOTS:
     void enablePrimitiveRestart()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisable(GL_PRIMITIVE_RESTART);
@@ -929,7 +830,7 @@ private Q_SLOTS:
     void enableVertexAttribute()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLVertexArrayObject vao;
@@ -958,7 +859,7 @@ private Q_SLOTS:
     void frontFace()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glFrontFace(GL_CW);
@@ -979,7 +880,7 @@ private Q_SLOTS:
     void getRenderBufferDimensions()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLuint renderBufferId = 0;
@@ -1002,7 +903,7 @@ private Q_SLOTS:
     void getTextureDimensions()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLTexture texture(QOpenGLTexture::Target2D);
@@ -1023,7 +924,7 @@ private Q_SLOTS:
     void pointSize()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glEnable(GL_PROGRAM_POINT_SIZE);
@@ -1046,7 +947,7 @@ private Q_SLOTS:
     void maxClipPlaneCount()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLint maxCount = -1;
@@ -1059,7 +960,7 @@ private Q_SLOTS:
     void programUniformBlock()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1087,7 +988,7 @@ private Q_SLOTS:
     void programAttributesAndLocations()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1118,7 +1019,7 @@ private Q_SLOTS:
     void programUniformsAndLocations()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1177,15 +1078,15 @@ private Q_SLOTS:
     void programShaderStorageBlock()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
-        // Not supported in 3.3
+        // Not supported in 3.2
     }
 
     void releaseFrameBufferObject()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLuint fboId;
@@ -1204,7 +1105,7 @@ private Q_SLOTS:
     void setMSAAEnabled()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisable(GL_MULTISAMPLE);
@@ -1228,7 +1129,7 @@ private Q_SLOTS:
     void setAlphaCoverageEnabled()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisable(GL_SAMPLE_ALPHA_TO_COVERAGE);
@@ -1252,7 +1153,7 @@ private Q_SLOTS:
     void setClipPlane()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // Deprecated in 3.3 core
     }
@@ -1260,7 +1161,7 @@ private Q_SLOTS:
     void setSeamlessCubemap()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisable(GL_TEXTURE_CUBE_MAP_SEAMLESS);
@@ -1282,7 +1183,7 @@ private Q_SLOTS:
     void setVerticesPerPatch()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisable(GL_TEXTURE_CUBE_MAP_SEAMLESS);
@@ -1370,7 +1271,7 @@ private Q_SLOTS:
     void uniformsByteSize()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QFETCH(QByteArray, fragShader);
@@ -1411,11 +1312,10 @@ private Q_SLOTS:
         m_func->glUseProgram(0);
     }
 
-
     void useProgram()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1447,7 +1347,8 @@ private Q_SLOTS:
     void vertexAttribDivisor()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
+        // Not available in 3.2
     }
 
     void vertexAttributePointer()
@@ -1508,7 +1409,7 @@ private Q_SLOTS:
     void glUniform1fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1534,7 +1435,7 @@ private Q_SLOTS:
     void glUniform2fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1561,7 +1462,7 @@ private Q_SLOTS:
     void glUniform3fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1588,7 +1489,7 @@ private Q_SLOTS:
     void glUniform4fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1615,7 +1516,7 @@ private Q_SLOTS:
     void glUniform1iv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1641,7 +1542,7 @@ private Q_SLOTS:
     void glUniform2iv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1668,7 +1569,7 @@ private Q_SLOTS:
     void glUniform3iv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1695,7 +1596,7 @@ private Q_SLOTS:
     void glUniform4iv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1722,7 +1623,7 @@ private Q_SLOTS:
     void glUniform1uiv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1748,7 +1649,7 @@ private Q_SLOTS:
     void glUniform2uiv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1775,7 +1676,7 @@ private Q_SLOTS:
     void glUniform3uiv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1802,7 +1703,7 @@ private Q_SLOTS:
     void glUniform4uiv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1829,7 +1730,7 @@ private Q_SLOTS:
     void glUniformMatrix2fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1856,7 +1757,7 @@ private Q_SLOTS:
     void glUniformMatrix3fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1883,7 +1784,7 @@ private Q_SLOTS:
     void glUniformMatrix4fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1910,7 +1811,7 @@ private Q_SLOTS:
     void glUniformMatrix2x3fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1937,7 +1838,7 @@ private Q_SLOTS:
     void glUniformMatrix3x2fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1964,7 +1865,7 @@ private Q_SLOTS:
     void glUniformMatrix2x4fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1991,7 +1892,7 @@ private Q_SLOTS:
     void glUniformMatrix4x2fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -2018,7 +1919,7 @@ private Q_SLOTS:
     void glUniformMatrix3x4fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -2045,7 +1946,7 @@ private Q_SLOTS:
     void glUniformMatrix4x3fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -2069,11 +1970,10 @@ private Q_SLOTS:
         m_func->glUseProgram(0);
     }
 
-
     void blitFramebuffer()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         GLint maxSamples;
         m_func->glGetIntegerv(GL_MAX_SAMPLES, &maxSamples);
@@ -2099,13 +1999,13 @@ private Q_SLOTS:
         m_func->glFramebufferTexture(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, fboTextures[1], 0);
 
         GLenum status = m_func->glCheckFramebufferStatus(GL_FRAMEBUFFER);
-        QCOMPARE(status, GLenum(GL_FRAMEBUFFER_COMPLETE));
+        QVERIFY(status == GL_FRAMEBUFFER_COMPLETE);
 
         m_func->glBindFramebuffer(GL_FRAMEBUFFER, fbos[0]);
         m_func->glFramebufferTexture(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, fboTextures[0], 0);
 
         status = m_func->glCheckFramebufferStatus(GL_FRAMEBUFFER);
-        QCOMPARE(status, GLenum(GL_FRAMEBUFFER_COMPLETE));
+        QVERIFY(status == GL_FRAMEBUFFER_COMPLETE);
 
         m_func->glEnable(GL_MULTISAMPLE);
         m_func->glClearColor(0.2f, 0.2f, 0.2f, 0.2f);
@@ -2219,7 +2119,7 @@ private Q_SLOTS:
     void drawBuffer()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         m_func->glGetError();
 
@@ -2237,7 +2137,7 @@ private Q_SLOTS:
     void readBuffer()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         m_func->glGetError();
 
@@ -2354,9 +2254,10 @@ private Q_SLOTS:
     void rasterMode()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 functions not supported");
 
         m_func->glGetError();
+
         m_glHelper.rasterMode(GL_FRONT_AND_BACK, GL_LINE);
 
         // THEN
@@ -2370,8 +2271,8 @@ private Q_SLOTS:
 private:
     QScopedPointer<QWindow> m_window;
     QOpenGLContext m_glContext;
-    GraphicsHelperGL3_3 m_glHelper;
-    QOpenGLFunctions_3_3_Core *m_func = nullptr;
+    GraphicsHelperGL3_2 m_glHelper;
+    QOpenGLFunctions_3_2_Core *m_func = nullptr;
     bool m_initializationSuccessful = false;
 };
 
@@ -2382,7 +2283,7 @@ int main(int argc, char *argv[])
 #ifdef TEST_SHOULD_BE_PERFORMED
     QGuiApplication app(argc, argv);
     app.setAttribute(Qt::AA_Use96Dpi, true);
-    tst_GraphicsHelperGL3_3 tc;
+    tst_GraphicsHelperGL3_2 tc;
     QTEST_SET_MAIN_SOURCE_PATH
     return QTest::qExec(&tc, argc, argv);
 #endif
@@ -2390,5 +2291,5 @@ int main(int argc, char *argv[])
 }
 
 #ifdef TEST_SHOULD_BE_PERFORMED
-#include "tst_graphicshelpergl3_3.moc"
+#include "tst_graphicshelpergl3_2.moc"
 #endif
