@@ -28,12 +28,17 @@
 
 #include <QtTest/QTest>
 #include <qbackendnodetester.h>
+#include <Qt3DCore/qpropertyupdatedchange.h>
+#include <Qt3DCore/private/qbackendnode_p.h>
 #include <Qt3DRender/private/shaderbuilder_p.h>
 #include <Qt3DRender/qshaderprogram.h>
 #include <Qt3DRender/qshaderprogrambuilder.h>
 #include "testrenderer.h"
+#include "testpostmanarbiter.h"
 
-Q_DECLARE_METATYPE(Qt3DRender::Render::ShaderBuilder::ShaderType)
+#include <QByteArray>
+
+Q_DECLARE_METATYPE(Qt3DRender::QShaderProgram::ShaderType)
 
 class tst_ShaderBuilder : public Qt3DCore::QBackendNodeTester
 {
@@ -74,8 +79,8 @@ private slots:
         // THEN
         QVERIFY(!shaderBuilder.isEnabled());
         QVERIFY(shaderBuilder.enabledLayers().isEmpty());
-        for (int i = 0; i <= Qt3DRender::Render::ShaderBuilder::Compute; i++) {
-            const auto type = static_cast<Qt3DRender::Render::ShaderBuilder::ShaderType>(i);
+        for (int i = 0; i <= Qt3DRender::QShaderProgram::Compute; i++) {
+            const auto type = static_cast<Qt3DRender::QShaderProgram::ShaderType>(i);
             QCOMPARE(shaderBuilder.shaderGraph(type), QUrl());
             QCOMPARE(shaderBuilder.shaderCode(type), QByteArray());
             QVERIFY(!shaderBuilder.isShaderCodeDirty(type));
@@ -138,9 +143,11 @@ private slots:
         // GIVEN
         QFETCH(Qt3DRender::QShaderProgramBuilder*, frontend);
         Qt3DRender::Render::ShaderBuilder backend;
+        TestRenderer renderer;
 
         // WHEN
-        simulateInitialization(frontend, &backend);
+        backend.setRenderer(&renderer);
+        simulateInitializationSync(frontend, &backend);
 
         // THEN
         QVERIFY(backend.isEnabled() == frontend->isEnabled());
@@ -152,29 +159,29 @@ private slots:
 
         QCOMPARE(backend.enabledLayers(), frontend->enabledLayers());
 
-        QCOMPARE(backend.shaderGraph(Qt3DRender::Render::ShaderBuilder::Vertex), frontend->vertexShaderGraph());
-        QCOMPARE(backend.shaderCode(Qt3DRender::Render::ShaderBuilder::Vertex), QByteArray());
-        QCOMPARE(backend.isShaderCodeDirty(Qt3DRender::Render::ShaderBuilder::Vertex), !frontend->vertexShaderGraph().isEmpty());
+        QCOMPARE(backend.shaderGraph(Qt3DRender::QShaderProgram::Vertex), frontend->vertexShaderGraph());
+        QCOMPARE(backend.shaderCode(Qt3DRender::QShaderProgram::Vertex), QByteArray());
+        QCOMPARE(backend.isShaderCodeDirty(Qt3DRender::QShaderProgram::Vertex), !frontend->vertexShaderGraph().isEmpty());
 
-        QCOMPARE(backend.shaderGraph(Qt3DRender::Render::ShaderBuilder::TessellationControl), frontend->tessellationControlShaderGraph());
-        QCOMPARE(backend.shaderCode(Qt3DRender::Render::ShaderBuilder::TessellationControl), QByteArray());
-        QCOMPARE(backend.isShaderCodeDirty(Qt3DRender::Render::ShaderBuilder::TessellationControl), !frontend->tessellationControlShaderGraph().isEmpty());
+        QCOMPARE(backend.shaderGraph(Qt3DRender::QShaderProgram::TessellationControl), frontend->tessellationControlShaderGraph());
+        QCOMPARE(backend.shaderCode(Qt3DRender::QShaderProgram::TessellationControl), QByteArray());
+        QCOMPARE(backend.isShaderCodeDirty(Qt3DRender::QShaderProgram::TessellationControl), !frontend->tessellationControlShaderGraph().isEmpty());
 
-        QCOMPARE(backend.shaderGraph(Qt3DRender::Render::ShaderBuilder::TessellationEvaluation), frontend->tessellationEvaluationShaderGraph());
-        QCOMPARE(backend.shaderCode(Qt3DRender::Render::ShaderBuilder::TessellationEvaluation), QByteArray());
-        QCOMPARE(backend.isShaderCodeDirty(Qt3DRender::Render::ShaderBuilder::TessellationEvaluation), !frontend->tessellationEvaluationShaderGraph().isEmpty());
+        QCOMPARE(backend.shaderGraph(Qt3DRender::QShaderProgram::TessellationEvaluation), frontend->tessellationEvaluationShaderGraph());
+        QCOMPARE(backend.shaderCode(Qt3DRender::QShaderProgram::TessellationEvaluation), QByteArray());
+        QCOMPARE(backend.isShaderCodeDirty(Qt3DRender::QShaderProgram::TessellationEvaluation), !frontend->tessellationEvaluationShaderGraph().isEmpty());
 
-        QCOMPARE(backend.shaderGraph(Qt3DRender::Render::ShaderBuilder::Geometry), frontend->geometryShaderGraph());
-        QCOMPARE(backend.shaderCode(Qt3DRender::Render::ShaderBuilder::Geometry), QByteArray());
-        QCOMPARE(backend.isShaderCodeDirty(Qt3DRender::Render::ShaderBuilder::Geometry), !frontend->geometryShaderGraph().isEmpty());
+        QCOMPARE(backend.shaderGraph(Qt3DRender::QShaderProgram::Geometry), frontend->geometryShaderGraph());
+        QCOMPARE(backend.shaderCode(Qt3DRender::QShaderProgram::Geometry), QByteArray());
+        QCOMPARE(backend.isShaderCodeDirty(Qt3DRender::QShaderProgram::Geometry), !frontend->geometryShaderGraph().isEmpty());
 
-        QCOMPARE(backend.shaderGraph(Qt3DRender::Render::ShaderBuilder::Fragment), frontend->fragmentShaderGraph());
-        QCOMPARE(backend.shaderCode(Qt3DRender::Render::ShaderBuilder::Fragment), QByteArray());
-        QCOMPARE(backend.isShaderCodeDirty(Qt3DRender::Render::ShaderBuilder::Fragment), !frontend->fragmentShaderGraph().isEmpty());
+        QCOMPARE(backend.shaderGraph(Qt3DRender::QShaderProgram::Fragment), frontend->fragmentShaderGraph());
+        QCOMPARE(backend.shaderCode(Qt3DRender::QShaderProgram::Fragment), QByteArray());
+        QCOMPARE(backend.isShaderCodeDirty(Qt3DRender::QShaderProgram::Fragment), !frontend->fragmentShaderGraph().isEmpty());
 
-        QCOMPARE(backend.shaderGraph(Qt3DRender::Render::ShaderBuilder::Compute), frontend->computeShaderGraph());
-        QCOMPARE(backend.shaderCode(Qt3DRender::Render::ShaderBuilder::Compute), QByteArray());
-        QCOMPARE(backend.isShaderCodeDirty(Qt3DRender::Render::ShaderBuilder::Compute), !frontend->computeShaderGraph().isEmpty());
+        QCOMPARE(backend.shaderGraph(Qt3DRender::QShaderProgram::Compute), frontend->computeShaderGraph());
+        QCOMPARE(backend.shaderCode(Qt3DRender::QShaderProgram::Compute), QByteArray());
+        QCOMPARE(backend.isShaderCodeDirty(Qt3DRender::QShaderProgram::Compute), !frontend->computeShaderGraph().isEmpty());
 
         // WHEN
         backend.cleanup();
@@ -182,8 +189,8 @@ private slots:
         // THEN
         QVERIFY(!backend.isEnabled());
         QVERIFY(backend.enabledLayers().isEmpty());
-        for (int i = 0; i <= Qt3DRender::Render::ShaderBuilder::Compute; i++) {
-            const auto type = static_cast<Qt3DRender::Render::ShaderBuilder::ShaderType>(i);
+        for (int i = 0; i <= Qt3DRender::QShaderProgram::Compute; i++) {
+            const auto type = static_cast<Qt3DRender::QShaderProgram::ShaderType>(i);
             QCOMPARE(backend.shaderGraph(type), QUrl());
             QCOMPARE(backend.shaderCode(type), QByteArray());
             QVERIFY(!backend.isShaderCodeDirty(type));
@@ -196,25 +203,14 @@ private slots:
     {
         // GIVEN
         Qt3DRender::Render::ShaderBuilder backend;
+        Qt3DRender::QShaderProgramBuilder frontend;
         TestRenderer renderer;
         backend.setRenderer(&renderer);
+        simulateInitializationSync(&frontend, &backend);
 
         // WHEN
-        auto updateChange = Qt3DCore::QPropertyUpdatedChangePtr::create(Qt3DCore::QNodeId());
-        updateChange->setValue(true);
-        updateChange->setPropertyName("enabled");
-        backend.sceneChangeEvent(updateChange);
-
-        // THEN
-        QVERIFY(backend.isEnabled());
-        QCOMPARE(renderer.dirtyBits(), Qt3DRender::Render::AbstractRenderer::ShadersDirty);
-        renderer.resetDirty();
-
-        // WHEN
-        updateChange = Qt3DCore::QPropertyUpdatedChangePtr::create(Qt3DCore::QNodeId());
-        updateChange->setValue(false);
-        updateChange->setPropertyName("enabled");
-        backend.sceneChangeEvent(updateChange);
+        frontend.setEnabled(false);
+        backend.syncFromFrontEnd(&frontend, false);
 
         // THEN
         QVERIFY(!backend.isEnabled());
@@ -222,10 +218,8 @@ private slots:
         renderer.resetDirty();
 
         // WHEN
-        updateChange = Qt3DCore::QPropertyUpdatedChangePtr::create(Qt3DCore::QNodeId());
-        updateChange->setValue(true);
-        updateChange->setPropertyName("enabled");
-        backend.sceneChangeEvent(updateChange);
+        frontend.setEnabled(true);
+        backend.syncFromFrontEnd(&frontend, false);
         // AND
         backend.cleanup();
 
@@ -239,26 +233,24 @@ private slots:
     {
         // GIVEN
         Qt3DRender::Render::ShaderBuilder backend;
+        Qt3DRender::QShaderProgramBuilder frontend;
         TestRenderer renderer;
         backend.setRenderer(&renderer);
-        const auto programId = Qt3DCore::QNodeId::createId();
+        simulateInitializationSync(&frontend, &backend);
 
         // WHEN
-        auto updateChange = Qt3DCore::QPropertyUpdatedChangePtr::create(Qt3DCore::QNodeId());
-        updateChange->setValue(QVariant::fromValue(programId));
-        updateChange->setPropertyName("shaderProgram");
-        backend.sceneChangeEvent(updateChange);
+        Qt3DRender::QShaderProgram prog;
+        frontend.setShaderProgram(&prog);
+        backend.syncFromFrontEnd(&frontend, false);
 
         // THEN
-        QCOMPARE(backend.shaderProgramId(), programId);
+        QCOMPARE(backend.shaderProgramId(), prog.id());
         QCOMPARE(renderer.dirtyBits(), Qt3DRender::Render::AbstractRenderer::ShadersDirty);
         renderer.resetDirty();
 
         // WHEN
-        updateChange = Qt3DCore::QPropertyUpdatedChangePtr::create(Qt3DCore::QNodeId());
-        updateChange->setValue(QVariant::fromValue(Qt3DCore::QNodeId()));
-        updateChange->setPropertyName("shaderProgram");
-        backend.sceneChangeEvent(updateChange);
+        frontend.setShaderProgram(nullptr);
+        backend.syncFromFrontEnd(&frontend, false);
 
         // THEN
         QVERIFY(backend.shaderProgramId().isNull());
@@ -266,10 +258,8 @@ private slots:
         renderer.resetDirty();
 
         // WHEN
-        updateChange = Qt3DCore::QPropertyUpdatedChangePtr::create(Qt3DCore::QNodeId());
-        updateChange->setValue(QVariant::fromValue(programId));
-        updateChange->setPropertyName("shaderProgram");
-        backend.sceneChangeEvent(updateChange);
+        frontend.setShaderProgram(&prog);
+        backend.syncFromFrontEnd(&frontend, false);
         // AND
         backend.cleanup();
 
@@ -282,27 +272,41 @@ private slots:
     void shouldHandleEnabledLayersPropertyChange()
     {
         // GIVEN
+        qputenv("QT3D_DISABLE_SHADER_CACHE", "1");
         Qt3DRender::Render::ShaderBuilder backend;
+        Qt3DRender::QShaderProgramBuilder frontend;
         TestRenderer renderer;
         backend.setRenderer(&renderer);
+        simulateInitializationSync(&frontend, &backend);
         const auto layers = QStringList() << "foo" << "bar";
 
-        for (int i = 0; i <= Qt3DRender::Render::ShaderBuilder::Compute; i++) {
-            const auto type = static_cast<Qt3DRender::Render::ShaderBuilder::ShaderType>(i);
+        static const std::pair<
+                Qt3DRender::QShaderProgram::ShaderType,
+                void (Qt3DRender::QShaderProgramBuilder::*)(const QUrl &)
+                >
+                shaderTypesToSetters[] = {
+            {Qt3DRender::QShaderProgram::Vertex, &Qt3DRender::QShaderProgramBuilder::setVertexShaderGraph},
+            {Qt3DRender::QShaderProgram::TessellationControl, &Qt3DRender::QShaderProgramBuilder::setTessellationControlShaderGraph},
+            {Qt3DRender::QShaderProgram::TessellationEvaluation, &Qt3DRender::QShaderProgramBuilder::setTessellationEvaluationShaderGraph},
+            {Qt3DRender::QShaderProgram::Geometry, &Qt3DRender::QShaderProgramBuilder::setGeometryShaderGraph},
+            {Qt3DRender::QShaderProgram::Fragment, &Qt3DRender::QShaderProgramBuilder::setFragmentShaderGraph},
+            {Qt3DRender::QShaderProgram::Compute, &Qt3DRender::QShaderProgramBuilder::setComputeShaderGraph},
+        };
+
+
+        for (auto it = std::begin(shaderTypesToSetters), end = std::end(shaderTypesToSetters); it != end; ++it) {
             const auto graphUrl = QUrl::fromEncoded("qrc:/input.json");
-            backend.setShaderGraph(type, graphUrl);
+            (frontend.*(it->second))(graphUrl);
         }
 
         // WHEN
-        auto updateChange = Qt3DCore::QPropertyUpdatedChangePtr::create(Qt3DCore::QNodeId());
-        updateChange->setValue(layers);
-        updateChange->setPropertyName("enabledLayers");
-        backend.sceneChangeEvent(updateChange);
+        frontend.setEnabledLayers(layers);
+        backend.syncFromFrontEnd(&frontend, false);
 
         // THEN
         QCOMPARE(backend.enabledLayers(), layers);
-        for (int i = 0; i <= Qt3DRender::Render::ShaderBuilder::Compute; i++) {
-            const auto type = static_cast<Qt3DRender::Render::ShaderBuilder::ShaderType>(i);
+        for (int i = 0; i <= Qt3DRender::QShaderProgram::Compute; i++) {
+            const auto type = static_cast<Qt3DRender::QShaderProgram::ShaderType>(i);
             QVERIFY(backend.isShaderCodeDirty(type));
             backend.generateCode(type); // Resets the dirty flag
         }
@@ -310,31 +314,27 @@ private slots:
         renderer.resetDirty();
 
         // WHEN
-        updateChange = Qt3DCore::QPropertyUpdatedChangePtr::create(Qt3DCore::QNodeId());
-        updateChange->setValue(layers);
-        updateChange->setPropertyName("enabledLayers");
-        backend.sceneChangeEvent(updateChange);
+        frontend.setEnabledLayers(layers);
+        backend.syncFromFrontEnd(&frontend, false);
 
         // THEN
         QCOMPARE(backend.enabledLayers(), layers);
-        for (int i = 0; i <= Qt3DRender::Render::ShaderBuilder::Compute; i++) {
-            const auto type = static_cast<Qt3DRender::Render::ShaderBuilder::ShaderType>(i);
+        for (int i = 0; i <= Qt3DRender::QShaderProgram::Compute; i++) {
+            const auto type = static_cast<Qt3DRender::QShaderProgram::ShaderType>(i);
             QVERIFY(!backend.isShaderCodeDirty(type));
             backend.generateCode(type); // Resets the dirty flag
         }
-        QCOMPARE(renderer.dirtyBits(), Qt3DRender::Render::AbstractRenderer::ShadersDirty);
+        QCOMPARE(renderer.dirtyBits(), 0);
         renderer.resetDirty();
 
         // WHEN
-        updateChange = Qt3DCore::QPropertyUpdatedChangePtr::create(Qt3DCore::QNodeId());
-        updateChange->setValue(QStringList());
-        updateChange->setPropertyName("enabledLayers");
-        backend.sceneChangeEvent(updateChange);
+        frontend.setEnabledLayers(QStringList());
+        backend.syncFromFrontEnd(&frontend, false);
 
         // THEN
         QVERIFY(backend.shaderProgramId().isNull());
-        for (int i = 0; i <= Qt3DRender::Render::ShaderBuilder::Compute; i++) {
-            const auto type = static_cast<Qt3DRender::Render::ShaderBuilder::ShaderType>(i);
+        for (int i = 0; i <= Qt3DRender::QShaderProgram::Compute; i++) {
+            const auto type = static_cast<Qt3DRender::QShaderProgram::ShaderType>(i);
             QVERIFY(backend.isShaderCodeDirty(type));
             backend.generateCode(type); // Resets the dirty flag
         }
@@ -342,17 +342,15 @@ private slots:
         renderer.resetDirty();
 
         // WHEN
-        updateChange = Qt3DCore::QPropertyUpdatedChangePtr::create(Qt3DCore::QNodeId());
-        updateChange->setValue(layers);
-        updateChange->setPropertyName("enabledLayers");
-        backend.sceneChangeEvent(updateChange);
+        frontend.setEnabledLayers(layers);
+        backend.syncFromFrontEnd(&frontend, false);
         // AND
         backend.cleanup();
 
         // THEN
         QVERIFY(backend.enabledLayers().isEmpty());
-        for (int i = 0; i <= Qt3DRender::Render::ShaderBuilder::Compute; i++) {
-            const auto type = static_cast<Qt3DRender::Render::ShaderBuilder::ShaderType>(i);
+        for (int i = 0; i <= Qt3DRender::QShaderProgram::Compute; i++) {
+            const auto type = static_cast<Qt3DRender::QShaderProgram::ShaderType>(i);
             QVERIFY(!backend.isShaderCodeDirty(type));
             backend.generateCode(type); // Resets the dirty flag
         }
@@ -362,64 +360,68 @@ private slots:
 
     void shouldHandleShaderGraphPropertiesChanges_data()
     {
-        QTest::addColumn<QByteArray>("property");
-        QTest::addColumn<Qt3DRender::Render::ShaderBuilder::ShaderType>("type");
+        QTest::addColumn<Qt3DRender::QShaderProgram::ShaderType>("type");
         QTest::addColumn<QUrl>("graphUrl");
 
-        QTest::newRow("vertex") << QByteArrayLiteral("vertexShaderGraph")
-                                << Qt3DRender::Render::ShaderBuilder::Vertex
+        QTest::newRow("vertex") << Qt3DRender::QShaderProgram::Vertex
                                 << QUrl::fromEncoded("qrc:/vertex.json");
 
-        QTest::newRow("tessControl") << QByteArrayLiteral("tessellationControlShaderGraph")
-                                     << Qt3DRender::Render::ShaderBuilder::TessellationControl
+        QTest::newRow("tessControl") << Qt3DRender::QShaderProgram::TessellationControl
                                      << QUrl::fromEncoded("qrc:/tesscontrol.json");
 
-        QTest::newRow("tessEval") << QByteArrayLiteral("tessellationEvaluationShaderGraph")
-                                  << Qt3DRender::Render::ShaderBuilder::TessellationEvaluation
+        QTest::newRow("tessEval") << Qt3DRender::QShaderProgram::TessellationEvaluation
                                   << QUrl::fromEncoded("qrc:/tesseval.json");
 
-        QTest::newRow("geometry") << QByteArrayLiteral("geometryShaderGraph")
-                                  << Qt3DRender::Render::ShaderBuilder::Geometry
+        QTest::newRow("geometry") << Qt3DRender::QShaderProgram::Geometry
                                   << QUrl::fromEncoded("qrc:/geometry.json");
 
-        QTest::newRow("fragment") << QByteArrayLiteral("fragmentShaderGraph")
-                                  << Qt3DRender::Render::ShaderBuilder::Fragment
+        QTest::newRow("fragment") << Qt3DRender::QShaderProgram::Fragment
                                   << QUrl::fromEncoded("qrc:/fragment.json");
 
-        QTest::newRow("compute") << QByteArrayLiteral("computeShaderGraph")
-                                 << Qt3DRender::Render::ShaderBuilder::Compute
+        QTest::newRow("compute") << Qt3DRender::QShaderProgram::Compute
                                  << QUrl::fromEncoded("qrc:/compute.json");
     }
 
     void shouldHandleShaderGraphPropertiesChanges()
     {
         // GIVEN
-        QFETCH(QByteArray, property);
-        QFETCH(Qt3DRender::Render::ShaderBuilder::ShaderType, type);
+        QFETCH(Qt3DRender::QShaderProgram::ShaderType, type);
         QFETCH(QUrl, graphUrl);
 
         Qt3DRender::Render::ShaderBuilder backend;
+        Qt3DRender::QShaderProgramBuilder frontend;
         TestRenderer renderer;
         backend.setRenderer(&renderer);
+        simulateInitializationSync(&frontend, &backend);
+
+        static const QHash<
+                Qt3DRender::QShaderProgram::ShaderType,
+                void (Qt3DRender::QShaderProgramBuilder::*)(const QUrl &)
+                >
+                shaderTypesToSetters = {
+            {Qt3DRender::QShaderProgram::Vertex, &Qt3DRender::QShaderProgramBuilder::setVertexShaderGraph},
+            {Qt3DRender::QShaderProgram::TessellationControl, &Qt3DRender::QShaderProgramBuilder::setTessellationControlShaderGraph},
+            {Qt3DRender::QShaderProgram::TessellationEvaluation, &Qt3DRender::QShaderProgramBuilder::setTessellationEvaluationShaderGraph},
+            {Qt3DRender::QShaderProgram::Geometry, &Qt3DRender::QShaderProgramBuilder::setGeometryShaderGraph},
+            {Qt3DRender::QShaderProgram::Fragment, &Qt3DRender::QShaderProgramBuilder::setFragmentShaderGraph},
+            {Qt3DRender::QShaderProgram::Compute, &Qt3DRender::QShaderProgramBuilder::setComputeShaderGraph},
+        };
 
         // WHEN
-        auto updateChange = Qt3DCore::QPropertyUpdatedChangePtr::create(Qt3DCore::QNodeId());
-        updateChange->setValue(QUrl());
-        updateChange->setPropertyName(property);
-        backend.sceneChangeEvent(updateChange);
+
+        (frontend.*(shaderTypesToSetters[type]))(QUrl());
+        backend.syncFromFrontEnd(&frontend, false);
 
         // THEN
         QCOMPARE(backend.shaderGraph(type), QUrl());
         QVERIFY(!backend.isShaderCodeDirty(type));
         QVERIFY(backend.shaderCode(type).isEmpty());
-        QCOMPARE(renderer.dirtyBits(), Qt3DRender::Render::AbstractRenderer::ShadersDirty);
+        QCOMPARE(renderer.dirtyBits(), 0);
         renderer.resetDirty();
 
         // WHEN
-        updateChange = Qt3DCore::QPropertyUpdatedChangePtr::create(Qt3DCore::QNodeId());
-        updateChange->setValue(graphUrl);
-        updateChange->setPropertyName(property);
-        backend.sceneChangeEvent(updateChange);
+        (frontend.*(shaderTypesToSetters[type]))(graphUrl);
+        backend.syncFromFrontEnd(&frontend, false);
 
         // THEN
         QCOMPARE(backend.shaderGraph(type), graphUrl);
@@ -429,10 +431,8 @@ private slots:
         renderer.resetDirty();
 
         // WHEN
-        updateChange = Qt3DCore::QPropertyUpdatedChangePtr::create(Qt3DCore::QNodeId());
-        updateChange->setValue(QUrl());
-        updateChange->setPropertyName(property);
-        backend.sceneChangeEvent(updateChange);
+        (frontend.*(shaderTypesToSetters[type]))(QUrl());
+        backend.syncFromFrontEnd(&frontend, false);
 
         // THEN
         QCOMPARE(backend.shaderGraph(type), QUrl());
@@ -442,10 +442,8 @@ private slots:
         renderer.resetDirty();
 
         // WHEN
-        updateChange = Qt3DCore::QPropertyUpdatedChangePtr::create(Qt3DCore::QNodeId());
-        updateChange->setValue(graphUrl);
-        updateChange->setPropertyName(property);
-        backend.sceneChangeEvent(updateChange);
+        (frontend.*(shaderTypesToSetters[type]))(graphUrl);
+        backend.syncFromFrontEnd(&frontend, false);
         // AND
         backend.cleanup();
 
@@ -459,23 +457,24 @@ private slots:
 
     void shouldHandleShaderCodeGeneration_data()
     {
-        QTest::addColumn<Qt3DRender::Render::ShaderBuilder::ShaderType>("type");
+        QTest::addColumn<Qt3DRender::QShaderProgram::ShaderType>("type");
 
-        QTest::newRow("vertex") << Qt3DRender::Render::ShaderBuilder::Vertex;
-        QTest::newRow("tessControl") << Qt3DRender::Render::ShaderBuilder::TessellationControl;
-        QTest::newRow("tessEval") << Qt3DRender::Render::ShaderBuilder::TessellationEvaluation;
-        QTest::newRow("geometry") << Qt3DRender::Render::ShaderBuilder::Geometry;
-        QTest::newRow("fragment") << Qt3DRender::Render::ShaderBuilder::Fragment;
-        QTest::newRow("compute") << Qt3DRender::Render::ShaderBuilder::Compute;
+        QTest::newRow("vertex") << Qt3DRender::QShaderProgram::Vertex;
+        QTest::newRow("tessControl") << Qt3DRender::QShaderProgram::TessellationControl;
+        QTest::newRow("tessEval") << Qt3DRender::QShaderProgram::TessellationEvaluation;
+        QTest::newRow("geometry") << Qt3DRender::QShaderProgram::Geometry;
+        QTest::newRow("fragment") << Qt3DRender::QShaderProgram::Fragment;
+        QTest::newRow("compute") << Qt3DRender::QShaderProgram::Compute;
     }
 
     void shouldHandleShaderCodeGeneration()
     {
         // GIVEN
+        qputenv("QT3D_DISABLE_SHADER_CACHE", "1");
         Qt3DRender::Render::ShaderBuilder::setPrototypesFile(":/prototypes.json");
         QVERIFY(!Qt3DRender::Render::ShaderBuilder::getPrototypeNames().isEmpty());
 
-        QFETCH(Qt3DRender::Render::ShaderBuilder::ShaderType, type);
+        QFETCH(Qt3DRender::QShaderProgram::ShaderType, type);
 
         const auto gl3Api = []{
             auto api = Qt3DRender::GraphicsApiFilterData();
@@ -494,7 +493,7 @@ private slots:
             return api;
         }();
 
-        const auto readCode = [](const QString &suffix) -> QString {
+        const auto readCode = [](const QString &suffix) -> QByteArray {
             const auto filePath = QStringLiteral(":/output.") + suffix;
             QFile file(filePath);
             if (!file.open(QFile::ReadOnly | QFile::Text))
@@ -541,6 +540,271 @@ private slots:
         QVERIFY(!backend.isShaderCodeDirty(type));
         QCOMPARE(backend.shaderCode(type), es2Code);
     }
+
+    void checkCodeUpdatedNotification_data()
+    {
+        QTest::addColumn<Qt3DRender::QShaderProgram::ShaderType>("type");
+        QTest::addColumn<Qt3DRender::QShaderProgram::ShaderType>("notificationType");
+
+        QTest::newRow("vertex") << Qt3DRender::QShaderProgram::Vertex << Qt3DRender::QShaderProgram::Vertex;
+        QTest::newRow("tessControl") << Qt3DRender::QShaderProgram::TessellationControl << Qt3DRender::QShaderProgram::TessellationControl;
+        QTest::newRow("tessEval") << Qt3DRender::QShaderProgram::TessellationEvaluation << Qt3DRender::QShaderProgram::TessellationEvaluation;
+        QTest::newRow("geometry") << Qt3DRender::QShaderProgram::Geometry << Qt3DRender::QShaderProgram::Geometry;
+        QTest::newRow("fragment") << Qt3DRender::QShaderProgram::Fragment << Qt3DRender::QShaderProgram::Fragment;
+        QTest::newRow("compute") << Qt3DRender::QShaderProgram::Compute << Qt3DRender::QShaderProgram::Compute;
+    }
+
+
+    void checkCodeUpdatedNotification()
+    {
+        // GIVEN
+        qputenv("QT3D_DISABLE_SHADER_CACHE", "1");
+        Qt3DRender::Render::ShaderBuilder::setPrototypesFile(":/prototypes.json");
+        QVERIFY(!Qt3DRender::Render::ShaderBuilder::getPrototypeNames().isEmpty());
+        QFETCH(Qt3DRender::QShaderProgram::ShaderType, type);
+        QFETCH(Qt3DRender::QShaderProgram::ShaderType, notificationType);
+        Q_UNUSED(notificationType)
+
+        const auto gl3Api = []{
+            auto api = Qt3DRender::GraphicsApiFilterData();
+            api.m_api = Qt3DRender::QGraphicsApiFilter::OpenGL;
+            api.m_profile = Qt3DRender::QGraphicsApiFilter::CoreProfile;
+            api.m_major = 3;
+            api.m_minor = 2;
+            return api;
+        }();
+
+        const auto readCode = [](const QString &suffix) -> QByteArray {
+            const auto filePath = QStringLiteral(":/output.") + suffix;
+            QFile file(filePath);
+            if (!file.open(QFile::ReadOnly | QFile::Text))
+                qFatal("File open failed: %s", qPrintable(filePath));
+            return file.readAll();
+        };
+
+        const auto gl3Code = readCode("gl3");
+
+        Qt3DRender::Render::ShaderBuilder backend;
+        TestArbiter arbiter;
+        Qt3DCore::QBackendNodePrivate::get(&backend)->setArbiter(&arbiter);
+
+
+        // WHEN
+        const auto graphUrl = QUrl::fromEncoded("qrc:/input.json");
+        backend.setShaderGraph(type, graphUrl);
+
+        // THEN
+        QCOMPARE(backend.shaderGraph(type), graphUrl);
+        QVERIFY(backend.isShaderCodeDirty(type));
+        QVERIFY(backend.shaderCode(type).isEmpty());
+
+        // WHEN
+        backend.setGraphicsApi(gl3Api);
+        backend.generateCode(type);
+
+        // THEN
+        QCOMPARE(backend.shaderGraph(type), graphUrl);
+        QVERIFY(!backend.isShaderCodeDirty(type));
+        QCOMPARE(backend.shaderCode(type), gl3Code);
+    }
+
+    void checkFileCaching()
+    {
+        // GIVEN
+        qunsetenv("QT3D_DISABLE_SHADER_CACHE");
+        QTemporaryDir cacheDir;
+
+        if (!cacheDir.isValid()) {
+            QSKIP("Unable to generate cache dir, skipping");
+            return;
+        }
+        const auto gl3Api = []{
+            auto api = Qt3DRender::GraphicsApiFilterData();
+            api.m_api = Qt3DRender::QGraphicsApiFilter::OpenGL;
+            api.m_profile = Qt3DRender::QGraphicsApiFilter::CoreProfile;
+            api.m_major = 3;
+            api.m_minor = 2;
+            return api;
+        }();
+        const auto gl2Api = []{
+            auto api = Qt3DRender::GraphicsApiFilterData();
+            api.m_api = Qt3DRender::QGraphicsApiFilter::OpenGL;
+            api.m_profile = Qt3DRender::QGraphicsApiFilter::NoProfile;
+            api.m_major = 2;
+            api.m_minor = 0;
+            return api;
+        }();
+        Qt3DRender::Render::ShaderBuilder::setPrototypesFile(":/prototypes.json");
+        qputenv("QT3D_WRITABLE_CACHE_PATH", cacheDir.path().toUtf8());
+
+        // THEN
+        QVERIFY(QDir(cacheDir.path()).entryList(QDir::Files).empty());
+        QByteArray hashKey;
+        {
+            // WHEN
+            Qt3DRender::Render::ShaderBuilder b;
+            b.setGraphicsApi(gl3Api);
+            const auto graphUrl = QUrl::fromEncoded("qrc:/input.json");
+            b.setShaderGraph(Qt3DRender::QShaderProgram::Vertex, graphUrl);
+            b.generateCode(Qt3DRender::QShaderProgram::Vertex);
+
+            // THEN
+            QCOMPARE(QDir(cacheDir.path()).entryList(QDir::Files).count(), 1);
+
+            hashKey = b.hashKeyForShaderGraph(Qt3DRender::QShaderProgram::Vertex);
+            QCOMPARE(hashKey.length(), 40);
+
+            QCOMPARE(QDir(cacheDir.path()).entryList(QDir::Files).first(),
+                     QString::fromUtf8(hashKey) + QLatin1String(".qt3d"));
+        }
+        {
+            // WHEN
+            Qt3DRender::Render::ShaderBuilder b;
+            b.setGraphicsApi(gl3Api);
+            const auto graphUrl = QUrl::fromEncoded("qrc:/input.json");
+            b.setShaderGraph(Qt3DRender::QShaderProgram::Vertex, graphUrl);
+            b.generateCode(Qt3DRender::QShaderProgram::Vertex);
+
+            // THEN
+            QCOMPARE(QDir(cacheDir.path()).entryList(QDir::Files).count(), 1);
+            QCOMPARE(QDir(cacheDir.path()).entryList(QDir::Files).first(),
+                     QString::fromUtf8(hashKey) + QLatin1String(".qt3d"));
+        }
+        {
+            // WHEN
+            Qt3DRender::Render::ShaderBuilder b;
+            b.setGraphicsApi(gl2Api);
+            const auto graphUrl = QUrl::fromEncoded("qrc:/input.json");
+            b.setShaderGraph(Qt3DRender::QShaderProgram::Vertex, graphUrl);
+            b.generateCode(Qt3DRender::QShaderProgram::Vertex);
+            QByteArray gl2HashKey = b.hashKeyForShaderGraph(Qt3DRender::QShaderProgram::Vertex);
+
+            // THEN
+            QCOMPARE(QDir(cacheDir.path()).entryList(QDir::Files).count(), 2);
+            QVERIFY(gl2HashKey != hashKey);
+        }
+    }
+
+    void checkRuntimeCaching()
+    {
+        // GIVEN
+        qunsetenv("QT3D_DISABLE_SHADER_CACHE");
+        TestRenderer renderer;
+        QTemporaryDir cacheDir;
+
+        if (!cacheDir.isValid()) {
+            QSKIP("Unable to generate cache dir, skipping");
+            return;
+        }
+        const auto gl3Api = []{
+            auto api = Qt3DRender::GraphicsApiFilterData();
+            api.m_api = Qt3DRender::QGraphicsApiFilter::OpenGL;
+            api.m_profile = Qt3DRender::QGraphicsApiFilter::CoreProfile;
+            api.m_major = 3;
+            api.m_minor = 2;
+            return api;
+        }();
+        Qt3DRender::Render::ShaderBuilder::setPrototypesFile(":/prototypes.json");
+        qputenv("QT3D_WRITABLE_CACHE_PATH", cacheDir.path().toUtf8());
+
+        // THEN
+        QVERIFY(QDir(cacheDir.path()).entryList(QDir::Files).empty());
+
+        // WHEN
+        Qt3DRender::Render::ShaderBuilder b;
+        b.setGraphicsApi(gl3Api);
+        const auto graphUrl = QUrl::fromEncoded("qrc:/input.json");
+        b.setShaderGraph(Qt3DRender::QShaderProgram::Vertex, graphUrl);
+        b.generateCode(Qt3DRender::QShaderProgram::Vertex);
+
+        // THEN
+        QCOMPARE(QDir(cacheDir.path()).entryList(QDir::Files).count(), 1);
+
+        const QByteArray hashKey = b.hashKeyForShaderGraph(Qt3DRender::QShaderProgram::Vertex);
+        QCOMPARE(hashKey.length(), 40);
+
+        QCOMPARE(QDir(cacheDir.path()).entryList(QDir::Files).first(),
+                 QString::fromUtf8(hashKey) + QLatin1String(".qt3d"));
+
+        QVERIFY(!renderer.containsGeneratedShaderGraph(hashKey));
+
+        // WHEN
+        b.setRenderer(&renderer);
+        b.generateCode(Qt3DRender::QShaderProgram::Vertex);
+
+        // THEN
+        QVERIFY(renderer.containsGeneratedShaderGraph(hashKey));
+    }
+
+    void checkDontUseCache()
+    {
+        // GIVEN
+        QTemporaryDir cacheDir;
+
+        if (!cacheDir.isValid()) {
+            QSKIP("Unable to generate cache dir, skipping");
+            return;
+        }
+        const auto gl3Api = []{
+            auto api = Qt3DRender::GraphicsApiFilterData();
+            api.m_api = Qt3DRender::QGraphicsApiFilter::OpenGL;
+            api.m_profile = Qt3DRender::QGraphicsApiFilter::CoreProfile;
+            api.m_major = 3;
+            api.m_minor = 2;
+            return api;
+        }();
+        Qt3DRender::Render::ShaderBuilder::setPrototypesFile(":/prototypes.json");
+
+        // THEN
+        QVERIFY(QDir(cacheDir.path()).entryList(QDir::Files).empty());
+
+        // WHEN
+        qputenv("QT3D_DISABLE_SHADER_CACHE", "1");
+        Qt3DRender::Render::ShaderBuilder b;
+        b.setGraphicsApi(gl3Api);
+        const auto graphUrl = QUrl::fromEncoded("qrc:/input.json");
+        b.setShaderGraph(Qt3DRender::QShaderProgram::Vertex, graphUrl);
+        b.generateCode(Qt3DRender::QShaderProgram::Vertex);
+
+        // THEN
+        QCOMPARE(QDir(cacheDir.path()).entryList(QDir::Files).count(), 0);
+    }
+
+    void checkForceRebuildCache()
+    {
+        // GIVEN
+        QTemporaryDir cacheDir;
+
+        if (!cacheDir.isValid()) {
+            QSKIP("Unable to generate cache dir, skipping");
+            return;
+        }
+        const auto gl3Api = []{
+            auto api = Qt3DRender::GraphicsApiFilterData();
+            api.m_api = Qt3DRender::QGraphicsApiFilter::OpenGL;
+            api.m_profile = Qt3DRender::QGraphicsApiFilter::CoreProfile;
+            api.m_major = 3;
+            api.m_minor = 2;
+            return api;
+        }();
+        Qt3DRender::Render::ShaderBuilder::setPrototypesFile(":/prototypes.json");
+
+        // THEN
+        QVERIFY(QDir(cacheDir.path()).entryList(QDir::Files).empty());
+
+        // WHEN
+        qputenv("QT3D_WRITABLE_CACHE_PATH", cacheDir.path().toUtf8());
+        qputenv("QT3D_DISABLE_SHADER_CACHE", "1");
+        qputenv("QT3D_REBUILD_SHADER_CACHE", "1");
+        Qt3DRender::Render::ShaderBuilder b;
+        b.setGraphicsApi(gl3Api);
+        const auto graphUrl = QUrl::fromEncoded("qrc:/input.json");
+        b.setShaderGraph(Qt3DRender::QShaderProgram::Vertex, graphUrl);
+        b.generateCode(Qt3DRender::QShaderProgram::Vertex);
+
+        // THEN -> We have rebuilt the shader file (even if we don't use it)
+        QCOMPARE(QDir(cacheDir.path()).entryList(QDir::Files).count(), 1);
+    }
 };
 
 QTEST_MAIN(tst_ShaderBuilder)
